#************************************************************************
#
# Author: Tarun Goyal      
#
# Module: statusbar_manager.tcl
#
# Descr.: StatusBarManager is a central repository which will entertain
#         request that is recieved from any window of the framework, for 
#         displaying the relevant widgets on the status bar.
#         
# All Right Reserved
#
#***********************************************************************/


# StatusBarManager is a central repository which will entertain
# request that is recieved from any window of the framework.

itcl::class StatusBarManager {
   #variable list
          
   # Following data-structures are maintained by Task-Manager internally ::
   
   ## all the widgets in the status bar
   private variable statusBarWidgets

   # these store the widget related properties
   private variable widgetProps
   
   # List of created widgets are maintained as ::
   # createdWidgets {} = { label, .................... }
   private variable  createdWidgets {}
          
          
   # Global list of registered doc-bars and buttons ::
   # Reg_WindowWidgets = {  windowName, ...........}
   # Reg_WindowWidgetCallback(widgetName) = {  CallbackFunction , ........... }
   private variable  Reg_WindowWidgets 
   private variable  Reg_WindowWidgetCallback
          
   # Status Bar associated with a window:
   # windowStatusBar(windowName) = {statusBarObject}
   private variable  windowStatusBar
  
   ## widgets getting registered
   private variable _statusBarVars
          
   # this will store dftFramework's object
   private variable frameworkHandle
  
   ## to store status-bar object
   private variable globalStatusBarObj

   private variable _columnIndex
    
   private variable _storedFocus

   #methods
   constructor { frameworkObj } {
      global _vars
      
      set frameworkHandle $frameworkObj
      
      set globalStatusBarObj [frame $_vars(debug_win)._bottomFrame]
       
      ## for the initial setting of widgets 
      initializeLists
     
      set index 0
      foreach widget $statusBarWidgets {
         set _columnIndex($widget) $index
         incr index
      }  
   }
   
   
   public method showStatusBar {} {
      
      grid $globalStatusBarObj -row 6 -column 1 -sticky ew -columnspan 6 -rowspan 1
      
   }
   
   # public methods
           
   #  Methods that will be used by child-windows
   # Register_Window::  This  function enables a window to register associated status bar widget
   public method  Register_Window  { windowName widgetName  {callBackFunc ""}}
          
   # this method return the value stored in the widget variable
   public method getWidgetVal { windowName widgetName }

   # this method sets the widget's variable value to "value", with "progressBarVal" is an
   # optional argument that will be valid in the case of "progressBar" widget.
   public method setWidgetVal { windowName widgetName value {progressBarText ""}}

   #  this  method  changes the state of widgets depending  upon
   # the current selection inside a window
   public method UpdateState { windowName widgetName State}
          
   #  Methods that will be used by framework
   # Update_StatusBar :: This proc is  called by dftFramework and  depending on the windowName updates the status-bar
   public method Update_StatusBar  { windowName Docked }
     
   # getUndockedstatusBarObj  :  This  proc will return object of the statusbar, that goes into the Undocked Window. 
   #This proc is called as soon as the window is undocked.
   public method GetUndockedStatusBarObj { windowName }
           
   # private methods
   # Create_StatusBar :: Create a status-bar with a given status-bar-name..
   private method Create_StatusBar { widgetName windowName }
          
   # this will handle the call-backs
   public method CallBackHandler { widgetName }

   public method setPages { {curPage 0} {totPages 0} }
   public method getPageEntry {}

   # this function initializes the various status-bar related Lists
   private method initializeLists {}

}

## FOLLOWING METHOD CONTAINS SOME EXAMPLE OF THE WAY WIDGETS COULD BE SPECFIED IN THE STATUS BAR MANAGER. 
itcl::body StatusBarManager::initializeLists {} {
   set statusBarWidgets { "progBar" "selObj" "objShown" "gateReport" "hieLevel" "gateLevel" "abstraction" "instGroup" "page" "wiresShown"}
  
   set widgetProps(selObj)  { "label" "normal" "sunken" "left"}
   set widgetProps(page)  { "entryWidget" "Sheet:" "numeric" }
   set widgetProps(progBar)  { "progressBar" }
    
}

 
itcl::body StatusBarManager::Register_Window { windowName widgetName {callBackFunc ""}} {
   set enterFlag 0
   
   # Firstly check for the validity of the three arguments
      
   # If the "windowName" does not exist in the "widgetName"
   # we add it to the "widgetName" as follows
   if  { ![info exists Reg_WindowWidgets($widgetName)] || \
         [lsearch $Reg_WindowWidgets($widgetName) $windowName]  == -1  } {
      
      set enterFlag 1
      
      ## here we add windowName to the list of widgets
      if { ![info exists Reg_WindowWidgets($widgetName)] } {
         lappend Reg_WindowWidgets($widgetName) $windowName 
      } elseif { [info exists Reg_WindowWidgets($widgetName)] && \
                 [lsearch $Reg_WindowWidgets($widgetName) $windowName] == -1 } {
         lappend Reg_WindowWidgets($widgetName) $windowName
      }

      # add the call-back function to the array
      lappend Reg_WindowWidgetCallback($widgetName) $callBackFunc
   }  
       
   # Now we see whether there already exists a widet
   # with the given "widgetName", if not, we create it
   if { $enterFlag == 1 } {
      if { [lsearch $createdWidgets $widgetName]  == -1 } {
         # if docBar not found
         lappend  createdWidgets $widgetName
      }
      Create_StatusBar $widgetName $windowName
   } 
}
 
 
itcl::body StatusBarManager::Update_StatusBar { windowName Docked } {
    
   if { $Docked == "docked" } {
      set widgetsManaged [grid slaves $globalStatusBarObj]
   } else {
      set widgetsManaged [grid slaves $windowStatusBar($windowName)]
   } 
   
   foreach widget $statusBarWidgets {
      if { [info exists Reg_WindowWidgets($widget)] } {
         if { [lsearch $Reg_WindowWidgets($widget) $windowName ] == -1 && \
              [lsearch $Reg_WindowWidgets($widget) "frameworkWindow" ] == -1 } { 
            if { [lsearch $widgetsManaged $_statusBarVars($widget)] != -1 } { 
               grid remove $_statusBarVars($widget)
               grid columnconfigure $globalStatusBarObj $_columnIndex($widget) -weight 0
            }
         } else {
            if { [lsearch $widgetsManaged $_statusBarVars($widget)] == -1 } { 
               grid $_statusBarVars($widget) -row 0 -column $_columnIndex($widget) -sticky ew
               if { [lindex $widgetProps($widget) 0] != "progressBar" && [lindex $widgetProps($widget) 0] != "entryWidget"  } {
                  grid columnconfigure $globalStatusBarObj $_columnIndex($widget) -weight 2
               } else {
                  grid columnconfigure $globalStatusBarObj $_columnIndex($widget) -weight 1
               }
            }
       
            ## clearing the status bar contents
            if { $windowName == "frameworkWindow" } {
               if { [lindex $widgetProps($widget) 0] == "progressBar" } {
                  setWidgetVal $windowName $widget 0
               } else {
                  setWidgetVal $windowName $widget ""
               }
            }; #end of $windowName == "frameworkWindow"
         }
      }
   }           
}  

  
itcl::body StatusBarManager::Create_StatusBar { widgetName windowName }  {
   global _vars
   global _fonts
   
   set underscore "_"
   global var$underscore$widgetName
   global var$underscore$widgetName$underscore$windowName
   set paneManager [$frameworkHandle getPaneManager]
   
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName"]
      } else {
         set windowState "normal" 
      }
   } else {
      set windowState "normal"
   }
   

   if { ($windowState != "undocked" && ![info exists _statusBarVars($widgetName)])  || 
        ($windowState == "undocked" && ![info exists _statusBarVars($windowName$widgetName)]) } {
  
      set commandString [list $this "CallBackHandler" $widgetName] 
      if { [lindex $widgetProps($widgetName) 0] == "label" } {
         if {  $windowState != "undocked" } {
            set _statusBarVars($widgetName) [label $globalStatusBarObj._$widgetName \
                                      -textvar var$underscore$widgetName  \
                                      -font $_fonts(DlgNormal) \
                                      -relief [lindex $widgetProps($widgetName) 2] \
                                      -state [lindex $widgetProps($widgetName) 1] \
                                      -justify [lindex $widgetProps($widgetName) 3] -anchor w -width 3]
         } else {
            set _statusBarVars($windowName$widgetName) [label $windowStatusBar($windowName)._$windowName$widgetName \
                                    -textvar var$underscore$widgetName$underscore$windowName -relief [lindex $widgetProps($widgetName) 2] \
                                    -font $_fonts(DlgNormal) \
                                    -state [lindex $widgetProps($widgetName) 1] \
                                    -justify [lindex $widgetProps($widgetName) 3] -anchor w -width 3]
            if {[catch {set hover_component($widgetName) [::mtiwidgets::mtiballoon $_statusBarVars($widgetName).hovermenu]}]} {
                  set hover_component($widgetName) $_statusBarVars($widgetName).hovermenu
            }
         }       
      } elseif { [lindex $widgetProps($widgetName) 0] == "entryWidget" } {
         if {  $windowState != "undocked" } {
            set _statusBarVars($widgetName) [iwidgets::entryfield $globalStatusBarObj._$widgetName \
                                      -textvariable var$underscore$widgetName  \
                                      -labeltext [lindex $widgetProps($widgetName) 1] \
                                      -labelfont $_fonts(DlgNormal) \
                                      -validate [lindex $widgetProps($widgetName) 2] \
                                      -command "$this getPageEntry" -width 6 ]
         } else {
            set _statusBarVars($windowName$widgetName) [iwidgets::entryfield $windowStatusBar($windowName)._$windowName$widgetName \
                                      -textvariable var$underscore$widgetName$underscore$windowName \
                                      -labeltext [lindex $widgetProps($widgetName) 1] \
                                      -labelfont $_fonts(DlgNormal) \
                                      -validate [lindex $widgetProps($widgetName) 2] \
                                      -command "$this getPageEntry" -width 6 ]
         }       
      } elseif { [lindex $widgetProps($widgetName) 0] == "progressBar" } {
         if {  $windowState != "undocked" } {
            set _statusBarVars($widgetName) [progmeter $globalStatusBarObj._$widgetName  -orient h \
                                        -bg white -fg maroon -lblfg black -bold -lbl_inside -fromStatusBar]
            [progmeter $_statusBarVars($widgetName) wget meter] configure -width 12 
         } else {
            set _statusBarVars($windowName$widgetName) [progmeter $windowStatusBar($windowName)._$windowName$widgetName -orient h \
                                        -bg white -fg maroon -lblfg black -bold -lbl_inside -fromStatusBar] 
            [progmeter $_statusBarVars($windowName$widgetName) wget meter] configure -width 12
         }
      }  
      
      ## gridding being done here 
      if {  $windowState != "undocked" } {
         grid $_statusBarVars($widgetName) -row 0 -column $_columnIndex($widgetName) -sticky ew
         if { [lindex $widgetProps($widgetName) 0] != "progressBar" && [lindex $widgetProps($widgetName) 0] != "entryWidget" } {
            grid columnconfigure $globalStatusBarObj $_columnIndex($widgetName) -weight 2
         } else {
            grid columnconfigure $globalStatusBarObj $_columnIndex($widgetName) -weight 1
         }    
      } else {
         grid $_statusBarVars($windowName$widgetName) -row 0 -column $_columnIndex($widgetName) -sticky ew
         if { [lindex $widgetProps($widgetName) 0] != "progressBar" && [lindex $widgetProps($widgetName) 0] != "entryWidget" } {
            grid columnconfigure $windowStatusBar($windowName) $_columnIndex($widgetName) -weight 2
         } else {
            grid columnconfigure $windowStatusBar($windowName) $_columnIndex($widgetName) -weight 1
         }
      }
   }
}

     
# this proc will create a new tool-bar-object for the
# given window name and returns it 
itcl::body StatusBarManager::GetUndockedStatusBarObj { windowName } {
   
   # if the status-bar object exits for the given window then,
   # return it, else create a new one
   if { ![info exists windowStatusBar($windowName)] } {
      set windowStatusBar($windowName) [frame "[$frameworkHandle getPaneManager].$windowName.statusbar" ]

      foreach widget $statusBarWidgets {
         if { [info exists Reg_WindowWidgets($widget)] } {
            if { [lsearch $Reg_WindowWidgets($widget) $windowName ] != -1 } {
               Create_StatusBar $widget $windowName
            }
         }
      }
   }

   return $windowStatusBar($windowName)   
} 

   
itcl::body StatusBarManager::CallBackHandler { widgetName } {
   set activeWindowTag "yes"

   # first we get the "windowName" which is currently selected
   set activeWindow [$frameworkHandle getCurrentChildWindow]
   
   set windowIndex [lsearch $Reg_WindowWidgets($widgetName) $activeWindow]
    
   if { $windowIndex == -1 }  {
      set activeWindowTag "no"
      ## setting the window-index for "frameworkWindow"
      set windowIndex  [lsearch $Reg_WindowWidgets($widgetName) "frameworkWindow" ]
   }

   # here we call the respective method
   set callBackData [lindex $Reg_WindowWidgetCallback($widgetName) $windowIndex]
   set functionName [lindex $callBackData 0]
   
   puts "callBackData = $callBackData"
   puts "functionName = $functionName"
   
   if { $activeWindowTag == "yes" } {
      set functionArgs "::DftDebugFramework::$activeWindow $functionName"
   } else {
      set functionArgs "$frameworkHandle $functionName"
   }

   if { [lreplace $callBackData 0 0 ] != "" } {
      set counter 1
      while { $counter < [llength $callBackData] } { 
        lappend functionArgs [lindex $callBackData $counter]
        incr counter
      }
   }
      
   puts "functionArgs = $functionArgs"
   catch {eval [join $functionArgs]}

} 
           
 
itcl::body StatusBarManager::UpdateState { state windowName widgetName } {
   
   set paneManager [$frameworkHandle getPaneManager]

   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }

   if { $windowState != "undocked" } {
      set widPath $_statusBarVars($widgetName)
   } else {
      set widPath $_statusBarVars($windowName$widgetName)
   }      
  
   ## setting the state
   $widPath configure -state $state 
}



## this function returns the value stored in the variable of widget
itcl::body StatusBarManager::getWidgetVal { windowName widgetName } {
   set underscore "_"
   
   ## radiobutton case needs to treated separately
   if { [lindex $widgetProps($widgetName) 0] != "progressBar" } {
      global var$underscore$widgetName
      global var$underscore$widgetName$underscore$windowName
   } 


   set paneManager [$frameworkHandle getPaneManager]

   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   } else {
      set windowState "normal"
   }
   

   
   if { $windowState != "undocked" } {
      if { [lindex $widgetProps($widgetName) 0] != "progressBar" } {
         set tempStore "var$underscore$widgetName"
      } 
   } else {
      if { [lindex $widgetProps($widgetName) 0] != "progressBar" } {
         set tempStore "var$underscore$widgetName$underscore$windowName"
      } 
   }
  
   set returnVal [set $tempStore]
   if { [lindex $widgetProps($widgetName) 0] != "progressBar" } {
      return $returnVal
   } 
}

## this function stores the "value" in the widget's variable
itcl::body StatusBarManager::setWidgetVal { windowName widgetName value {progressBarText ""}} {
   global _fonts
   set underscore "_"
   global _vars
   
   ## radiobutton case needs to treated separately
   if { [lindex $widgetProps($widgetName) 0] != "progressBar" } {
      global var$underscore$widgetName
      global var$underscore$widgetName$underscore$windowName
       
      if { [lindex $widgetProps($widgetName) 0] == "label" } {
         if { [info exists _statusBarVars($widgetName)] && $value != "" } { 
            if {[catch {set hover_component($widgetName) [::mtiwidgets::mtiballoon $_statusBarVars($widgetName).hovermenu]}]} {
               set hover_component($widgetName) $_statusBarVars($widgetName).hovermenu
            }
            $hover_component($widgetName) balloonbindings  $_statusBarVars($widgetName) $value
         } 
      }
   } 


   set paneManager [$frameworkHandle getPaneManager]
 
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         if { [catch { set windowState [$paneManager state "$paneManager.$windowName" ] } ] } {
            set windowName "transcript"
            $frameworkHandle ShowWindow "transcript"
         }
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }
   
   if { $windowState != "undocked" } {
      if { [lindex $widgetProps($widgetName) 0] != "progressBar" } {
         set var$underscore$widgetName $value
         set var$underscore$widgetName$underscore$windowName $value

      } else {
         if { $value != 100 && $value != 0 && [focus] != "" } {
            #Commenting this code because grab focus has been moved on Stop button. Gaurav Bansal Apr 21, 2010
            #catch { grab set [focus] }
            #set _storedFocus [focus]
         } else {
            if { [info exists _storedFocus] && ( $value == 100 || $value == 0) } {
               #catch { grab release $_storedFocus }
               #catch {unset _storedFocus}
            }
         }

         if { $progressBarText != "" } {
            set meterWid [progmeter $_statusBarVars($widgetName) wget meter]
            ## passing percentage here
            progmeter $_statusBarVars($widgetName) -pct $value -fromStatusBar
            ## setting text here 
            $meterWid itemconf meter_pct -text $progressBarText -font $_fonts(DlgNormal)
            update 
         } else {
            progmeter $_statusBarVars($widgetName) -pct $value -fromStatusBar
            update 
         }
      }         
   } else {
      if { [lindex $widgetProps($widgetName) 0] != "progressBar" } { 
            set var$underscore$widgetName$underscore$windowName $value
      } else {
         if { $progressBarText != "" } {
            set meterWid [progmeter $_statusBarVars($windowName$widgetName) wget meter]
            ## passing percentage here
            progmeter $_statusBarVars($windowName$widgetName) -pct $value -fromStatusBar
            ## setting text here
            $meterWid itemconf meter_pct -text $progressBarText -font $_fonts(DlgNormal)
            update 
         } else {
            catch {progmeter $_statusBarVars($windowName$widgetName) -pct $value -fromStatusBar}
            update 
         }
      }
   } 
   
   if { [info exists _vars(forceTagDelDes)] && ($_vars(forceTagDelDes) == 1) && ( [lindex $widgetProps($widgetName) 0] == "progressBar" ) } {
       set meterWid [progmeter $_statusBarVars($widgetName) wget meter]
       ## passing percentage here
       progmeter $_statusBarVars($widgetName) -pct 100 -fromStatusBar
       ## setting text here 
       $meterWid itemconf meter_pct -text "Only TRANSCRIPT Active Till New Design Read AND \"TOP\" Set" -font $_fonts(DlgNormal) 
   }

}
  

itcl::body StatusBarManager::setPages { {curPage 0} {totPages 0} } {
   
   #setWidgetVal "debug" "page" "" 
   #setWidgetVal "design" "page" "" 
   
   set activeWindow [$frameworkHandle getCurrentChildWindow]
   
   setWidgetVal $activeWindow "page" ""
   if { $curPage > 0 && $curPage <= $totPages } {
      #setWidgetVal "design" "page" "$curPage of $totPages"
      #setWidgetVal "debug"  "page" "$curPage of $totPages"
      setWidgetVal $activeWindow "page" "$curPage of $totPages"
   }
}

itcl::body StatusBarManager::getPageEntry {} {
   
   set activeWindow [$frameworkHandle getCurrentChildWindow]
   set pageVal [getWidgetVal $activeWindow "page"]
   ::DftDebugFramework::$activeWindow DisplayPage $pageVal

}



