
#************************************************************************
#
# Author: Tarun Goyal    
#
# Module: toolbar_manager.tcl
#
# Descr.: MenuBarManager is a central repository which will entertain
#         request that is recieved from any window of the framework, for 
#         displaying the relevant dockbars/buttons/widgets on the toolbar
#         
# All Rights Reserved
#
#***********************************************************************/


itcl::class ToolBarManager {
   #variable list
          
   # Following data-structures are maintained by Task-Manager internally ::
   
   # these store the dock-button related properties
   private variable globalDockBars
   private variable dockBarButtons
   private variable dockBarDisplayName
   private variable buttonProps
   
   
   # List of created doc-bars are maintained as ::
   # createsDocBars {} = { std, .................... }
   private variable  createdDocBars {}
          
   # Array which tells us about the status of a dock-bar ::
   # docBarStatus (std) = { "Enable" } ...............
   private variable  docBarStatus
   private variable hideForceTag       
   # Global list of registered doc-bars and buttons ::
   # Reg_WindowDocBar (docName) = {  windowName, ...........}
   # Reg_WindowButton (docName, ButtonName) = { windowName, ........................}  
   # Reg_WindButtonCallback(docName, ButtonName) = {  CallbackFunction , ........... }
   private variable  Reg_WindowDocBar 
   private variable  Reg_WindowButton
   private variable  Reg_WindButtonCallback
          
   # Tool Bar associated with a window:
   # windowToolBar(windowName) = {toolBarObject}
   private variable  windowToolBar
         
   # this will store the global tool-bar object of the main framework
   private  variable  globalToolBarObject       
   
   # this will store dftFramework's object
   private variable frameworkHandle
    
   #methods
   constructor { toolBarName frameworkObj } {
      global _vars
      
      set frameworkHandle $frameworkObj

      ## call to initialize various lists
      initializeLists
      
      set globalToolBarObject [mtiwidgets::dockbar $_vars(debug_win).$toolBarName -relief sunken -borderwidth 1]
      foreach dockBar $globalDockBars {
         set docBarStatus($dockBar) ""
         set hideForceTag($dockBar) "false"
      }
   }
   
   # this function initializes the various dock-bar related Lists
   private method initializeLists {}
   
   # Register_Window::  This  function enables a window to register associated 
   # doc-Name and button-Name
   public method  Register_Window  { windowName docName ButtonName {callBackFunc ""}}
          
   # Unregister_Window:: This function deregisters the given window from the 
   # both the lists :: Reg_WindowDocBar and Reg_WindowButton.
   public method  Unregister_Window {windowName }

   # Update_ToolBar :: This proc is  called by dftFramework and  depending on
   # the windowName updates the respective toolbar. Showing/hiding the toolbar 
   # is also handled here.
   public method Update_ToolBar  { windowName Docked { hideDockbar "" }}
     
   # Update_DocBar :: This function updates the requested dock-bar, with the 
   # latest-status of the buttons residing inside the dock-bar.
   private method  Update_DocBar { docBar_Name object windowName }

   # Create_DocBar :: Create a dock-bar with a given doc-bar-name..
   private method  Create_DocBar { docBarName toolBarObj windowName}
 
   #  Create_Button :: Create a button in the specified doc-bar, with the given
   # button-name
   private method  Create_Button { docBarName buttonName docBarObj windowName { enableFlag 1 } }
      
   # getUndockedToolBarObj  :  This  window will return object of the toolbar, 
   # that goes into the Undocked Window. This proc is called as soon as the window is undocked.
   public method GetUndockedToolBarObj { windowName  }
          
   # this will handle the call-backs
   public method CallBackHandler { buttonName docBarName}
          
   #  this  method  changes the state of buttons  depending  upon
   # the current selection inside a window
   public method UpdateState { state windowName docBarName { buttonName "" } }
   
   #  this  method  returns the state of buttons
   public method GetState { windowName docBarName { buttonName "" } }
   
   # this method handles tool-bars when a window is maximized
   public method maximizeWindowToolBars { windowName }
   
   # this method return the value stored in the widget variable
   public method getWidgetVal { windowName dockBarName buttonName }

   # this method sets the widget's variable value to "value"
   public method setWidgetVal { windowName dockBarName buttonName value {comboWidget ""}}
   
   # return global tool bar object
   public method getGlobalToolBarObj {} {
      return $globalToolBarObject
   }
  
   # this is for setting the state of a dock-bar
   public method setDockBarState { windowName dockBarName state }
   
   public method getcomponent { dockBarName buttonName }

  }

itcl::body ToolBarManager::initializeLists {} {
   set globalDockBars                { standard edit instances instData hieData zoom find}
   set dockBarButtons(standard)      { print open save copy undo redo refresh stop}
   set dockBarButtons(edit)          { cut paste selectAll }
   set dockBarButtons(instances)     { addInst reportGates gateLevel traceOne traceEnd delSel delAll }
   set dockBarButtons(instData)      { noData testSetup loadUnload shift stateStab errorPatt pattIndex optNcp uoptNcp falsepath}
   set dockBarButtons(hieData)       { gates primitives totFaults undetFaults faultByCat drcByCat testCov relTestCov testCovLoss faultCov atpgEffec clkAtt}
   set dockBarButtons(zoom)          { zoomAll zoomIn zoomOut zoomSel zoomMark }   
   set dockBarButtons(find)          { entryBox localFind globalFind}

   set dockBarDisplayName(standard)         "Standard"
   set dockBarDisplayName(edit)             "Edit"
   

   set buttonProps(standard,print)  { "Print..." "print.gif" "Print" "button"}
   set buttonProps(standard,open)  { "Open File..." "new.gif" "Open File" "button"}
   set buttonProps(standard,save)    { "Save..." "save.gif" "Save" "button"}
   
   
   # set buttonProps(zoom,navigate) { "normal" "yes" "Sheet:" "combobox"}
   
   # set buttonProps(find,entryBox) { "normal" "Find:" "25" "entry"}
   # set buttonProps(find,localFind) { "Active Window Search" "find.gif" "Local Window Search" "button"}
  # set buttonProps(find,globalFind) { "Global Search" "global_search.gif" "Global Search" "button"}
}

 
itcl::body ToolBarManager::Register_Window { windowName docName buttonName {callBackFunc ""}} {
   set enterFlag 0
   
   # Firstly check for the validity of the three arguments
      
   # If the "windowName" does not exist in the "docName"
   # we add it to the "docName" as follows
   if  { ![info exists Reg_WindowDocbar($docName)] || \
         [lsearch $Reg_WindowDocBar($docName) $windowName]  == -1  } {
      
      set enterFlag 1
      
      if { ![info exists Reg_WindowDocBar($docName)] } {
         lappend Reg_WindowDocBar($docName) $windowName 
      } elseif { [info exists Reg_WindowDocBar($docName)] && \
                 [lsearch $Reg_WindowDocBar($docName) $windowName] == -1 } {
         lappend Reg_WindowDocBar($docName) $windowName
      }

      #next we add the button name to the respective window name
      lappend Reg_WindowButton($docName,$buttonName) $windowName
            
      # add the call-back function to the array
      lappend Reg_WindButtonCallback($docName,$buttonName) $callBackFunc
   }  
       
   # Now we see whether there already exists a  dock-bar
   # with the given "docbarName", if not, we create it
   if { $enterFlag == 1 } {
      if { [lsearch $createdDocBars $docName]  == -1 } {
         # if docBar not found
         lappend  createdDocBars $docName
      }
      Create_DocBar $docName $globalToolBarObject $windowName
   } 
}
 
itcl::body ToolBarManager::Unregister_Window { windowName } {
   
   if { [info exists windowToolBar($windowName)] } { 
      ## window is docked
      set object $windowToolBar($windowName)
      itcl::delete object $object
      set $windowToolBar($windowName) ""
   } else {
      ## window is docked
      set object $globalToolBarObject
      #  Unregister the "WindowName" by taking it out
      #  from the list Reg_WindowButton & Reg_WindowDocBar
      foreach docBar $globalDockBars {
         if { [info exists Reg_WindowDocBar($docBar)] } {
            set index  [lsearch $Reg_WindowDocBar($docBar) $windowName]
            if { $index != -1 } {
               # search the windowName over the entire button-list
               foreach buttonName $dockBarButtons($docBar) {
                  if { [info exists Reg_WindowButton($docBar,$buttonName) ] } {
                     set button_index [lsearch $Reg_WindowButton($docBar,$buttonName) $windowName]          
                     if {  $button_index != -1} {
                        set  Reg_WindowButton($docBar,$buttonName) \
                        [lreplace $Reg_WindowButton($docBar,$buttonName) $button_index $button_index]
                     }
                  }
               }
               set  Reg_WindowDocBar($docBar) [lreplace $Reg_WindowDocBar($docBar) $index $index]
            }
        
            #  Delete the "doc-bar" object, if we have emptied the list
            if { [llength $Reg_WindowDocBar($docBar)] == 0 } {
               set index 0
               set docBarName [ $object itemname $index ]
               while {$docBarName != ""} {
                  if { $docBarName == $docBar } {
                     # delete the respective object
                     $object remove $index 
                  }
               incr index 1
               set docBarName [ $object itemname $index ]
               }
            }
         }
      } ; #end of foreach   
   }
   Update_ToolBar $windowName "docked" 
}
 
itcl::body ToolBarManager::Update_ToolBar { windowName Docked  { hideDocBar "" } } {
   
   # this will have the dock-bars that needs to be deactivated
   set notNeededDockBars {} 
   set paneManager [$frameworkHandle getPaneManager]

   #  Here we update the toolbar asoociated with the
   #  window "windowName" , that has undergone a
   #  change both for docked and un-docked window
     
   if { $Docked == "docked" } {
      set obj $globalToolBarObject

   } else {
      set obj $windowToolBar($windowName)
   }
   
   #  Get the list of all dockbars associated with this
   #  window and see the status of associated doc-bars
   # and display them accordingly
   foreach dockBarName $globalDockBars {
      set isDocBarNeeded FALSE
      # check for if window is docked and dockbar" is needed
      if { [info exists Reg_WindowDocBar($dockBarName)] } {
         if { ( $Docked == "docked" ) && ( [lsearch $Reg_WindowDocBar($dockBarName) $windowName] != -1  ||  \
               [lsearch $Reg_WindowDocBar($dockBarName) "frameworkWindow" ] != -1 ) } {
            set  isDocBarNeeded TRUE
         } elseif { [lsearch $Reg_WindowDocBar($dockBarName) $windowName] != -1 } {
            # window is undocked but dockbar needed    
            set  isDocBarNeeded TRUE
         } else {
            lappend notNeededDockBars $dockBarName
         }
      }    
      
      
      ## here we find the index for the given 
      ## doc-bar name
      set index 0
      set end [ $obj index end]
      set reqIndex ""
      while {$index <= $end} {
         set docName [ $obj itemname $index ]
         if { $dockBarName == $docName } {
            # set the respective index
            set reqIndex $index
            set dockPath [$obj component $dockBarName]
            break
         }
         incr index 
      }
     
      # here we check whether there is a need
      # to hide the dock-bar
      set hideTag 1
      if { [info exists Reg_WindowDocBar($dockBarName) ] } {
         foreach window $Reg_WindowDocBar($dockBarName) {
            if { ( $window != "frameworkWindow" ) && \
                 ([$paneManager state "$paneManager.$window" ] != "hidden" && \
                  [$paneManager state "$paneManager.$window" ] != "undocked")  } {
               set hideTag 0
            } elseif { $window == "frameworkWindow" } {
               set hideTag 0
            }
         }
      }
      
      # special case of "hiding" the dock-bar
      if { ($hideTag == 1 && $docBarStatus($dockBarName) != "hide") || \
            $hideForceTag($dockBarName) == "true" } {      
         ## hide the dock-bar
         $obj hide $reqIndex   
         set docBarStatus($dockBarName) "hide"
         set hideTag 1
      }
      
      if { $isDocBarNeeded == "TRUE" && $hideTag == 0 } {
         if { $reqIndex != ""  } {
            ## show the dock-bar
            $obj show $reqIndex
            $dockPath configure -state normal
            set docBarStatus($dockBarName) "enabled"

            Update_DocBar $dockBarName $dockPath $windowName
         } elseif { $reqIndex == "" && $docBarStatus($dockBarName) != "disabled" } {
            ## disable the dock-bar
            set dockPath [$obj component $dockBarName]
            $dockPath configure -state disabled
            set docBarStatus($dockBarName) "disabled"
         }    
      } elseif { $hideTag == 0  && $Docked == "docked" } {
         if { $docBarStatus($dockBarName) != "disabled" } {
            # disable the dock-bar
            set dockPath [$obj component $dockBarName]
            $dockPath configure -state disabled
            set docBarStatus($dockBarName) "disabled"
         } 
      }
   } ; # end of foreach
}  

itcl::body ToolBarManager::Update_DocBar { docBarName object windowName } {
   
   set paneManager [$frameworkHandle getPaneManager]
   
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   } else {
      set windowState "normal"
   }

   set end [$object index end]

   # show the dockbar with the latest button-status
   foreach buttonName $dockBarButtons($docBarName) {
      if { ( [ info exists Reg_WindowButton($docBarName,$buttonName) ] ) } {
         
         ## if window is undocked there is a need of an 
         ## extra check of "frameworkWindow"
         if { ( $windowState != "undocked" ) && \
              ( [lsearch $Reg_WindowButton($docBarName,$buttonName) $windowName] != -1 || \
              [lsearch $Reg_WindowButton($docBarName,$buttonName) "frameworkWindow"] != -1 ) } {
            # that button was registed, hence Enable this button
            set index 0
            while {$index <= $end} {
               set tempButtonName [$object itemname $index]
               if { $tempButtonName == $buttonName } {
                  [$object component $buttonName] configure -state normal
                  break
               }
               incr index
            }
         } elseif { ( $windowState == "undocked" ) && \
                    ( [lsearch $Reg_WindowButton($docBarName,$buttonName) $windowName] != -1 ) } {
            # that button was registed, hence Enable this button
            set index 0
            while {$index <= $end} {
               set tempButtonName [$object itemname $index]
               if { $tempButtonName == $buttonName } {
                  [$object component $buttonName] configure -state normal
                  break
               }
               incr index
            }
         } else {
            # Disable the button
            set index 0
            while {$index <= $end} {
               set tempButtonName [$object itemname $index]
               if { $tempButtonName == $buttonName } {
                  [$object component $buttonName] configure -state disabled
                  break
               }
               incr index
            }
         }
      } else {
         # Disable the button
         set index 0
         while {$index <= $end} {
            set tempButtonName [$object itemname $index]
            if { $tempButtonName == $buttonName } {
               [$object component $buttonName] configure -state disabled
               break
            }
            incr index
         }
      }
   }
}

  
itcl::body ToolBarManager::Create_DocBar { docBarName toolBarObj windowName }  {
   global _vars
   set dupFlag 0
   
   # check for the dockname, whether it exists already or not
   set index 0
   set end [$toolBarObj index end]
   while {$index <= $end} {
      set docName [ $toolBarObj itemname $index ]
      if { $docName == $docBarName } {
         set dupFlag 1
         set dockBarObj [ $toolBarObj component $docBarName ]
      }
      incr index
   }   
   
   ## add dockbar if it does not exists already
   if { $dupFlag == 0 } {
      set dockBarObj [$toolBarObj add $docBarName -wrap 0 -orient horizontal -helpvariable statusVar -displayname $dockBarDisplayName($docBarName)] 
   }
   
   #  here we traverse list of all  button-names associated
   #  with the given doc-bar
   foreach buttonName $dockBarButtons($docBarName)  {
       if { [info exists Reg_WindowButton($docBarName,$buttonName)] } {
          Create_Button $docBarName $buttonName $dockBarObj $windowName 1
       } else {
          Create_Button $docBarName $buttonName $dockBarObj $windowName 0
       }
   }
}
     
# this function creates the button "buttonName"
# in the given dock-bar "docBarName"
itcl::body ToolBarManager::Create_Button { docBarName buttonName docBarObj windowName { enableFlag 1 } } {
   global _vars
   global _fonts
   
   set imagedir "$_vars(TCL_BITMAP_SRC)"
   set underscore "_"
   global var$underscore$docBarName$underscore$buttonName
   global var$underscore$docBarName$underscore$buttonName$underscore$windowName
   set paneManager [$frameworkHandle getPaneManager]
   
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   } else {
      set windowState "normal"
   }
   
   ## here we are checking for the existence of the button
   set index 0
   set dupFlag 0
   set end [$docBarObj index end]
   while {$index <= $end} {
      set tempButtonName [$docBarObj itemname $index]
      if { $tempButtonName == $buttonName } {
         set dupFlag 1
         set buttonObject [$docBarObj component $buttonName ]
      }
      incr index
   }

   ## add the button if it does not exists
   set commandString [list $this "CallBackHandler" $buttonName $docBarName]
   if { $dupFlag == 0 } {
      if { [lindex $buttonProps($docBarName,$buttonName) 3] == "button" } {
         $docBarObj add button $buttonName \
          -helpstr [lindex $buttonProps($docBarName,$buttonName) 0] \
          -image [image create photo -file [file join $imagedir [lindex $buttonProps($docBarName,$buttonName) 1]]] \
          -balloonstr [lindex $buttonProps($docBarName,$buttonName) 2] \
          -command $commandString
      } elseif { [lindex $buttonProps($docBarName,$buttonName) 3] == "entry" } {
         if { $windowState != "undocked" } {
            $docBarObj add iwidgets::entryfield $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -labeltext [lindex $buttonProps($docBarName,$buttonName) 1] \
              -labelfont $_fonts(DlgNormal) \
              -width [lindex $buttonProps($docBarName,$buttonName) 2] \
              -command $commandString \
              -textvariable "var$underscore$docBarName$underscore$buttonName" 
                              
            #scope ::DftDebugFramework [set var$underscore$docBarName$underscore$buttonName $::DftDebugFramework::initVal]
            ## ${@itcl ::DftDebugFramework initVal}
         } else {
            $docBarObj add iwidgets::entryfield $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -labeltext [lindex $buttonProps($docBarName,$buttonName) 1] \
              -labelfont $_fonts(DlgNormal) \
              -width [lindex $buttonProps($docBarName,$buttonName) 2] \
              -command $commandString \
              -textvariable "var$underscore$docBarName$underscore$buttonName$underscore$windowName" 
         } 
      } elseif { [lindex $buttonProps($docBarName,$buttonName) 3] == "label" } {
           $docBarObj add label $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -text  [lindex $buttonProps($docBarName,$buttonName) 1] \
              -font $_fonts(DlgNormal)
      } elseif { [lindex $buttonProps($docBarName,$buttonName) 3] == "checkbutton" } {
         if { $windowState != "undocked" } {
            $docBarObj add checkbutton $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -text [lindex $buttonProps($docBarName,$buttonName) 1] \
              -font $_fonts(DlgNormal) \
              -variable "var$underscore$docBarName$underscore$buttonName" \
              -command $commandString \
              -width 2
         } else {
            $docBarObj add checkbutton $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -text [lindex $buttonProps($docBarName,$buttonName) 1]  \
              -font $_fonts(DlgNormal) \
              -variable "var$underscore$docBarName$underscore$buttonName$underscore$windowName" \
              -command $commandString \
              -width 2
         } 
      } elseif { [lindex $buttonProps($docBarName,$buttonName) 3] == "radiobutton" } {
         set buttonTail [lindex $buttonProps($docBarName,$buttonName) 4]
         if { $windowState != "undocked" } {
            $docBarObj add radiobutton $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -text [lindex $buttonProps($docBarName,$buttonName) 1] \
              -font $_fonts(DlgNormal) \
              -value [lindex $buttonProps($docBarName,$buttonName) 2] \
              -command $commandString \
              -variable "var$underscore$docBarName$underscore$buttonTail"
         } else {
            $docBarObj add radiobutton $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -text [lindex $buttonProps($docBarName,$buttonName) 1] \
              -font $_fonts(DlgNormal) \
              -value [lindex $buttonProps($docBarName,$buttonName) 2] \
              -command $commandString \
              -variable "var$underscore$docBarName$underscore$buttonTail$underscore$windowName"
         } 
      } elseif { [lindex $buttonProps($docBarName,$buttonName) 3] == "combobox" } {
         if { $windowState != "undocked" } {
            set var$underscore$docBarName$underscore$buttonName [ \
             $docBarObj add ::iwidgets::combobox $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -editable [lindex $buttonProps($docBarName,$buttonName) 1] \
              -labeltext [lindex $buttonProps($docBarName,$buttonName) 2] \
              -labelfont $_fonts(DlgNormal) \
              -selectioncommand $commandString \
              -validate numeric \
              -command $commandString  
            ] 
         } else {
            set var$underscore$docBarName$underscore$buttonName$underscore$windowName [ \
             $docBarObj add ::iwidgets::combobox $buttonName \
              -state [lindex $buttonProps($docBarName,$buttonName) 0] \
              -editable [lindex $buttonProps($docBarName,$buttonName) 1] \
              -labeltext [lindex $buttonProps($docBarName,$buttonName) 2] \
              -labelfont $_fonts(DlgNormal) \
              -selectioncommand $commandString \
              -validate numeric \
              -command $commandString
             ]
         } 
      }

      ## setting the state of the button right
      if { $enableFlag == 1 } {
         [$docBarObj component $buttonName] configure -state normal
      } else {
         [$docBarObj component $buttonName] configure -state disabled
      }   
   } elseif { $dupFlag == 1  && $enableFlag == 1 } {
      $buttonObject configure -state normal
   }
}
                            
# this proc will create a new tool-bar-object for the
# given window name and returns it 
itcl::body ToolBarManager::GetUndockedToolBarObj { windowName } {
   global _vars
   
   # if the tool-bar object exits for the given window then,
   # return it, else create a new one
   if { ![info exists windowToolBar($windowName)] } {
      set object [mtiwidgets::dockbar "[$frameworkHandle getPaneManager].$windowName.toolbar" -relief sunken -borderwidth 1]
         
      # next we see what all "new" docs bars will go into the window
      foreach  docName $globalDockBars {
         if { [info exists Reg_WindowDocBar($docName)] && \
               [lsearch $Reg_WindowDocBar($docName) $windowName ] != -1 }  {
            Create_DocBar $docName $object $windowName;       #this dock-bar will go into tool-bar
         }         
      }
      set windowToolBar($windowName) $object
   }
   return $windowToolBar($windowName)   
} 

   
itcl::body ToolBarManager::CallBackHandler { buttonName docBarName } {
   set activeWindowTag "yes"

   # first we get the "windowName" which is currently selected
   set activeWindow [$frameworkHandle getCurrentChildWindow]
   
   set windowIndex [lsearch $Reg_WindowButton($docBarName,$buttonName) $activeWindow]
    
   if { $windowIndex == -1 }  {
      set activeWindowTag "no"
      ## setting the window-index for "frameworkWindow"
      set windowIndex  [lsearch $Reg_WindowButton($docBarName,$buttonName) "frameworkWindow" ]
   }

   # here we call the respective method
   set callBackData [lindex $Reg_WindButtonCallback($docBarName,$buttonName) $windowIndex]
   set functionName [lindex $callBackData 0]
   
   if { $activeWindowTag == "yes" } {
      set functionArgs "::DftDebugFramework::$activeWindow $functionName"
   } else {
      set functionArgs "$frameworkHandle $functionName"
   }

   if { [lreplace $callBackData 0 0 ] != "" } {
      set counter 1
      while { $counter < [llength $callBackData] } { 
        lappend functionArgs [lindex $callBackData $counter]
        incr counter
      }
   }
      
   catch {eval [join $functionArgs]}

} 
           
 
itcl::body ToolBarManager::GetState { windowName docBarName { buttonName "" } } {
   
   # here we first find all the doc bars associated
   # with "windowName"
   
   set paneManager [$frameworkHandle getPaneManager]

   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }
   
   if { $windowState != "undocked" } {
      set object $globalToolBarObject
   } else {
      if { ![info exists windowToolBar($windowName)] } {
         return
      } 
      set object $windowToolBar($windowName)
   }
   
 
   ## setting the dock-bar-object here
   set dockPath [$object component $docBarName]
   set end [$dockPath index end]
   
   if { $buttonName != "" } { 
      if { [lsearch $Reg_WindowDocBar($docBarName) $windowName] != -1  && \
           [lsearch $Reg_WindowButton($docBarName,$buttonName) $windowName] != -1 } {
         set index 0
         while {$index <= $end} {
            set tempButtonName [$dockPath itemname $index]
            if { $tempButtonName == $buttonName } {
               set state [[$dockPath component $buttonName] cget -state]
               break
            }
            incr index
         }    
     }
   } else {
      if { [info exists Reg_WindowDocBar($docBarName)] && \
           [lsearch $Reg_WindowDocBar($docBarName) $windowName] != -1 } {
         set state [$dockPath cget -state]
      }
   }
   
   if { $state == "normal" } {
      set state "active"
   } 
   return $state
}


itcl::body ToolBarManager::UpdateState { state windowName docBarName { buttonName "" } } {
   
   # here we first find all the doc bars associated
   # with "windowName" and send an "update_DocBar" it
   if { $state == "active" } {
      set state "normal"
   } 
    set paneManager [$frameworkHandle getPaneManager]

   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }
   
   if { $windowState != "undocked" } {
      set object $globalToolBarObject
   } else {
      if { ![info exists windowToolBar($windowName)] } {
         return
      } 
      set object $windowToolBar($windowName)
   }
   
 
   ## setting the dock-bar-object here
   set dockPath [$object component $docBarName]
   set end [$dockPath index end]
   
   ## if buttonName is null then there is a call to enable/disable dockBar
   if { $buttonName != "" } { 
      if { [lsearch $Reg_WindowDocBar($docBarName) $windowName] != -1  && \
           [lsearch $Reg_WindowButton($docBarName,$buttonName) $windowName] != -1 } {
         set index 0
         while {$index <= $end} {
            set tempButtonName [$dockPath itemname $index]
            if { $tempButtonName == $buttonName } {
               [$dockPath component $buttonName] configure -state $state
               break
            }
            incr index
         }    
     }
   } else {
      if { [info exists Reg_WindowDocBar($docBarName)] && \
           [lsearch $Reg_WindowDocBar($docBarName) $windowName] != -1 } {
         $dockPath configure -state $state
      }
   }
   
}


itcl::body ToolBarManager::maximizeWindowToolBars { windowName } {
   
   ## here we highlight all the dock-bars that is associated
   ## with the "windowName"
   
   foreach dockName $globalDockBars {
      if { [info exists Reg_WindowDocBar($dockName)] && \
           [lsearch $Reg_WindowDocBar($dockName) $windowName] == -1 && \
           [lsearch $Reg_WindowDocBar($dockName) "frameworkWindow" ] == -1 } {
         
         ## here we find the index for the given 
         ## doc-bar name
         set index 0
         set end [ $globalToolBarObject index end]
         set reqIndex ""
         while {$index <= $end} {
            set dockBarName [ $globalToolBarObject itemname $index ]
            if { $dockBarName == $dockName } {
               # set the respective index
               set reqIndex $index
               break
            }
            incr index 
         }
         ## hiding the dock-bar
         $globalToolBarObject hide $reqIndex
      } elseif { [info exists Reg_WindowDocBar($dockName)] && [lsearch $Reg_WindowDocBar($dockName) $windowName] != -1 } {
         ## here we are enabling the dock-bar
         set dockPath [$globalToolBarObject component $dockName]
         $dockPath configure -state normal
         Update_DocBar $dockName $dockPath $windowName 
         
      }
   }
}

## this function returns the value stored in the variable of widget
itcl::body ToolBarManager::getWidgetVal { windowName dockBarName buttonName } {
   set underscore "_"
   
   ## radiobutton case needs to treated separately
   if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "radiobutton" } {
      global var$underscore$dockBarName$underscore$buttonName
      global var$underscore$dockBarName$underscore$buttonName$underscore$windowName
   } else {
      set buttonTail [lindex $buttonProps($dockBarName,$buttonName) 4]
      global var$underscore$dockBarName$underscore$buttonTail
      global var$underscore$dockBarName$underscore$buttonTail$underscore$windowName
   }


   set paneManager [$frameworkHandle getPaneManager]

   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   } else {
      set windowState "normal"
   }
   
   
   if { $windowState != "undocked" } {
      if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "radiobutton" } {
         set tempStore "var$underscore$dockBarName$underscore$buttonName"
      } else {
         set tempStore "var$underscore$dockBarName$underscore$buttonTail"
      }
   } else {
      if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "radiobutton" } {
         set tempStore "var$underscore$dockBarName$underscore$buttonName$underscore$windowName"
      } else {
         set tempStore "var$underscore$dockBarName$underscore$buttonTail$underscore$windowName" 
      }
   }
  
   set returnVal [set $tempStore]
   if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "combobox" } {
      return $returnVal
   } else {
      return [$returnVal get]
   }     

}

## this function stores the "value" in the widget's variable
itcl::body ToolBarManager::setWidgetVal { windowName dockBarName buttonName value {comboWidget ""}} {
   set underscore "_"
   
   ## radiobutton case needs to treated separately
   if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "radiobutton" } {
      global var$underscore$dockBarName$underscore$buttonName
      global var$underscore$dockBarName$underscore$buttonName$underscore$windowName
   } else {
      set buttonTail [lindex $buttonProps($dockBarName,$buttonName) 4]
      global var$underscore$dockBarName$underscore$buttonTail
      global var$underscore$dockBarName$underscore$buttonTail$underscore$windowName
   }


   set paneManager [$frameworkHandle getPaneManager]
   set comboTag 0
 
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }
   
   if { $windowState != "undocked" } {
      if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "combobox" } {
         if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "radiobutton" } {
            set var$underscore$dockBarName$underscore$buttonName $value
         } else {
            set var$underscore$dockBarName$underscore$buttonTail $value
         }  
      } 
      
      ## special treatment for combobox 
      if { [lindex $buttonProps($dockBarName,$buttonName) 3] == "combobox" } {
         set comboVal "var$underscore$dockBarName$underscore$buttonName"
         set comboTag 1         
      }  
   } else {
      if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "combobox" } { 
         if { [lindex $buttonProps($dockBarName,$buttonName) 3] != "radiobutton" } {   
            set var$underscore$dockBarName$underscore$buttonName$underscore$windowName $value
         } else {
            set var$underscore$dockBarName$underscore$buttonTail$underscore$windowName $value
         }
      } 
     
      ## special treatment for combobox 
      if { [lindex $buttonProps($dockBarName,$buttonName) 3] == "combobox" } {
         set comboVal "var$underscore$dockBarName$underscore$buttonName$underscore$windowName"
         set comboTag 1         
      }
   }
   
   if { $comboTag == 1 } {
      set comboPath [set $comboVal]
      if { $comboWidget == "list" } {
         $comboPath delete list 0 end
         for {set x 0} {$x<[llength $value]} {incr x} {
            $comboPath insert list end [lindex $value $x]
         }
      } elseif { $comboWidget == "entry" } {
         $comboPath clear entry
         $comboPath insert entry 0 $value
      }
   }
   
}
 


itcl::body ToolBarManager::setDockBarState { windowName dockBarName state } {
   set paneManager [$frameworkHandle getPaneManager]
   
   ## setting the window state here
   if { $windowName != "frameworkWindow" } {
      set windowManaged [$paneManager panes]
      if { $windowManaged != "" } {
         set windowState [ $paneManager state "$paneManager.$windowName" ]
      } else {
         set windowState "normal" 
      }
   }  else {
      set windowState "normal"
   }
   
   if { $windowState == "undocked" } {
      set obj $windowToolBar($windowName)
   } else {
      set obj $globalToolBarObject
   }

   ## here we find the index for the given 
   ## doc-bar name
   set index 0
   set end [ $obj index end]
   set reqIndex ""
   set required 0

   while {$index <= $end} {
      set docName [ $obj itemname $index ]
      if { $dockBarName == $docName } {
         # set the respective index
         set reqIndex $index
         break
      }
      incr index 
   }
    
   # setting the state of the dock-bar as per the request
   if { $reqIndex != "" } {      
      ## hide the dock-bar
      if { $state == "hide" } {
         set hideForceTag($dockBarName) "true"
         $obj $state $reqIndex 
      } else {
         set hideForceTag($dockBarName) "false"
         
         ## following is for checking if the dock-bar is valid for 
         ## any of the windows open
         foreach window [$frameworkHandle getChildWindowList] {
            set windName [$window getChildWindowName]
            if {  [ $paneManager state "$paneManager.$windName" ] == "normal" } {
               if {  [lsearch $Reg_WindowDocBar($dockBarName) $windName] != -1 } {
                  set required 1
               }
            }
         }

         if { [lsearch $Reg_WindowDocBar($dockBarName) $windowName] != -1  || $required == 1 || \
              [lsearch $Reg_WindowDocBar($dockBarName) "frameworkWindow" ] != -1  } {
               $obj $state $reqIndex
         }  
      }
   }  
      
   if { $windowState == "undocked" } {
      after idle "itcl::code Update_ToolBar $windowName $windowState"
   } else {
      foreach kid [$frameworkHandle getChildWindowList] {
         if { [$kid getChildWindowName] != $windowName } {
            itcl::code Update_ToolBar [$kid getChildWindowName] "docked"
         }
      }
      Update_ToolBar $windowName "docked"
   }
   catch {::DftDebugFramework::$windowName activate; }
}

itcl::body ToolBarManager::getcomponent { dockBarName buttonName } {
   if {![info exists buttonProps($dockBarName,$buttonName)]} {
      return ""
   }

   set dockPath [$globalToolBarObject component $dockBarName]
   set componentPath [$dockPath component $buttonName]
   return $componentPath
}
